from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum
from cpl_discord.service.discord_collection_abc import DiscordCollectionABC

from bot_core.abc.module_abc import ModuleABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from modules.base.abc.base_helper_abc import BaseHelperABC
from modules.base.command.afk_command import AFKCommand
from modules.base.command.game_server_group import GameServerGroup
from modules.base.command.help_command import HelpCommand
from modules.base.command.info_command import InfoCommand
from modules.base.command.mass_move_command import MassMoveCommand
from modules.base.command.ping_command import PingCommand
from modules.base.command.presence_command import PresenceCommand
from modules.base.command.purge_command import PurgeCommand
from modules.base.command.register_group import RegisterGroup
from modules.base.command.unregister_group import UnregisterGroup
from modules.base.command.user_group import UserGroup
from modules.base.events.base_on_command_error_event import BaseOnCommandErrorEvent
from modules.base.events.base_on_command_event import BaseOnCommandEvent
from modules.base.events.base_on_member_join_event import BaseOnMemberJoinEvent
from modules.base.events.base_on_member_remove_event import BaseOnMemberRemoveEvent
from modules.base.events.base_on_message_delete_event import BaseOnMessageDeleteEvent
from modules.base.events.base_on_message_event import BaseOnMessageEvent
from modules.base.events.base_on_raw_reaction_add import BaseOnRawReactionAddEvent
from modules.base.events.base_on_raw_reaction_remove import BaseOnRawReactionRemoveEvent
from modules.base.events.base_on_scheduled_event_update_event import BaseOnScheduledEventUpdateEvent
from modules.base.events.base_on_voice_state_update_event import (
    BaseOnVoiceStateUpdateEvent,
)
from modules.base.events.base_on_voice_state_update_event_help_channel import (
    BaseOnVoiceStateUpdateEventHelpChannel,
)
from modules.base.events.base_on_voice_state_update_event_scheduled_event_bonus import (
    BaseOnVoiceStateUpdateEventScheduledEventBonus,
)
from modules.base.helper.base_reaction_handler import BaseReactionHandler
from modules.base.service.base_helper_service import BaseHelperService
from modules.base.service.event_service import EventService
from modules.base.service.user_warnings_service import UserWarningsService


class BaseModule(ModuleABC):
    def __init__(self, dc: DiscordCollectionABC):
        ModuleABC.__init__(self, dc, FeatureFlagsEnum.base_module)

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        pass

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_transient(BaseHelperABC, BaseHelperService)
        services.add_transient(BaseReactionHandler)
        services.add_singleton(EventService)
        services.add_transient(UserWarningsService)

        # commands
        self._dc.add_command(AFKCommand)
        self._dc.add_command(HelpCommand)
        self._dc.add_command(InfoCommand)
        self._dc.add_command(MassMoveCommand)
        self._dc.add_command(PingCommand)
        self._dc.add_command(PresenceCommand)

        self._dc.add_command(PurgeCommand)
        self._dc.add_command(UserGroup)
        self._dc.add_command(RegisterGroup)
        self._dc.add_command(UnregisterGroup)
        self._dc.add_command(GameServerGroup)
        # events
        self._dc.add_event(DiscordEventTypesEnum.on_command.value, BaseOnCommandEvent)
        self._dc.add_event(DiscordEventTypesEnum.on_command_error.value, BaseOnCommandErrorEvent)
        self._dc.add_event(DiscordEventTypesEnum.on_member_join.value, BaseOnMemberJoinEvent)
        self._dc.add_event(DiscordEventTypesEnum.on_member_remove.value, BaseOnMemberRemoveEvent)
        self._dc.add_event(DiscordEventTypesEnum.on_message.value, BaseOnMessageEvent)
        self._dc.add_event(DiscordEventTypesEnum.on_message_delete.value, BaseOnMessageDeleteEvent)
        self._dc.add_event(DiscordEventTypesEnum.on_raw_reaction_add.value, BaseOnRawReactionAddEvent)
        self._dc.add_event(
            DiscordEventTypesEnum.on_raw_reaction_remove.value,
            BaseOnRawReactionRemoveEvent,
        )
        self._dc.add_event(
            DiscordEventTypesEnum.on_voice_state_update.value,
            BaseOnVoiceStateUpdateEvent,
        )
        self._dc.add_event(
            DiscordEventTypesEnum.on_voice_state_update.value,
            BaseOnVoiceStateUpdateEventHelpChannel,
        )
        self._dc.add_event(
            DiscordEventTypesEnum.on_voice_state_update.value,
            BaseOnVoiceStateUpdateEventScheduledEventBonus,
        )
        self._dc.add_event(
            DiscordEventTypesEnum.on_scheduled_event_update.value,
            BaseOnScheduledEventUpdateEvent,
        )
