import discord
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.helper.command_checks import CommandChecks
from bot_core.logging.command_logger import CommandLogger


class PresenceCommand(DiscordCommandABC):
    def __init__(
        self,
        logger: CommandLogger,
        message_service: MessageServiceABC,
        bot: DiscordBotServiceABC,
        translate: TranslatePipe,
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._t = translate

    @commands.hybrid_command()
    @commands.guild_only()
    @CommandChecks.check_is_ready()
    @CommandChecks.check_is_member_moderator()
    async def presence(self, ctx: Context, text: str = ""):
        self._logger.debug(__name__, f"Received command presence {ctx}")

        if text == "":
            await self._bot.change_presence(activity=None)
            await self._message_service.send_ctx_msg(ctx, self._t.transform("modules.base.presence.removed"))
            return

        if len(text) > 128:
            await self._message_service.send_ctx_msg(
                ctx, self._t.transform("modules.base.presence.max_char_count_exceeded")
            )
            return

        await self._bot.change_presence(activity=discord.Game(name=text))
        await self._message_service.send_ctx_msg(ctx, self._t.transform("modules.base.presence.changed"))

        self._logger.trace(__name__, f"Finished presence command")
